@@ -9,13 +9,16 @@ use std::net::ToSocketAddrs;
 use std::path::PathBuf;
 use std::str::FromStr;
 
+use std::env::consts::{ARCH, OS};
 use std::time::Duration;
 
+pub const ELECTRS_VERSION: &str = env!("CARGO_PKG_VERSION");
 const DEFAULT_SERVER_ADDRESS: [u8; 4] = [127, 0, 0, 1]; // by default, serve on IPv4 localhost
 
 mod internal {
     #![allow(unused)]
     #![allow(clippy::identity_conversion)]
+    #![allow(clippy::cognitive_complexity)]
 
     include!(concat!(env!("OUT_DIR"), "/configure_me_config.rs"));
 }
@@ -133,9 +136,11 @@ pub struct Config {
     pub jsonrpc_timeout: Duration,
     pub index_batch_size: usize,
     pub index_lookup_limit: Option<usize>,
+    pub reindex_last_blocks: usize,
     pub auto_reindex: bool,
     pub ignore_mempool: bool,
     pub sync_once: bool,
+    pub disable_electrum_rpc: bool,
     pub server_banner: String,
     pub args: Vec<String>,
 }
@@ -305,13 +310,18 @@ impl Config {
             jsonrpc_timeout: Duration::from_secs(config.jsonrpc_timeout_secs),
             index_batch_size: config.index_batch_size,
             index_lookup_limit,
+            reindex_last_blocks: config.reindex_last_blocks,
             auto_reindex: config.auto_reindex,
             ignore_mempool: config.ignore_mempool,
             sync_once: config.sync_once,
+            disable_electrum_rpc: config.disable_electrum_rpc,
             server_banner: config.server_banner,
             args: args.map(|a| a.into_string().unwrap()).collect(),
         };
-        eprintln!("{:?}", config);
+        eprintln!(
+            "Starting electrs {} on {} {} with {:?}",
+            ELECTRS_VERSION, ARCH, OS, config
+        );
         env_logger::Builder::from_default_env()
             .default_format()
             .format_timestamp_millis()
