@@ -4,6 +4,7 @@ use bitcoin::{
     hashes::hex::{FromHex, ToHex},
     BlockHash, Txid,
 };
+use crossbeam_channel::Receiver;
 use rayon::prelude::*;
 use serde_derive::Deserialize;
 use serde_json::{self, json, Value};
@@ -13,19 +14,17 @@ use std::iter::FromIterator;
 
 use crate::{
     cache::Cache,
-    config::Config,
+    config::{Config, ELECTRS_VERSION},
     daemon::{self, extract_bitcoind_error, Daemon},
     merkle::Proof,
-    metrics::Histogram,
+    metrics::{self, Histogram},
     signals::Signal,
     status::ScriptHashStatus,
     tracker::Tracker,
     types::ScriptHash,
 };
 
-const ELECTRS_VERSION: &str = env!("CARGO_PKG_VERSION");
 const PROTOCOL_VERSION: &str = "1.4";
-
 const UNKNOWN_FEE: isize = -1; // (allowed by Electrum protocol)
 
 /// Per-client Electrum protocol state
@@ -123,28 +122,37 @@ pub struct Rpc {
 
 impl Rpc {
     /// Perform initial index sync (may take a while on first run).
-    pub fn new(config: &Config, mut tracker: Tracker) -> Result<Self> {
-        let rpc_duration =
-            tracker
-                .metrics()
-                .histogram_vec("rpc_duration", "RPC duration (in seconds)", "method");
+    pub fn new(config: &Config) -> Result<Self> {
+        let tracker = Tracker::new(&config)?;
+        let rpc_duration = tracker.metrics().histogram_vec(
+            "rpc_duration",
+            "RPC duration (in seconds)",
+            "method",
+            metrics::default_duration_buckets(),
+        );
 
         let signal = Signal::new();
-        tracker
-            .sync(&Daemon::connect(config)?, signal.exit_flag())
-            .context("initial sync failed")?;
-
+        let daemon = Daemon::connect(config, signal.exit_flag(), tracker.metrics())?;
+        let cache = Cache::new(tracker.metrics());
         Ok(Self {
             tracker,
-            cache: Cache::default(),
+            cache,
             rpc_duration,
-            daemon: Daemon::connect(config)?,
+            daemon,
             signal,
             banner: config.server_banner.clone(),
             port: config.electrum_rpc_addr.port(),
         })
     }
 
+    pub(crate) fn signal(&self) -> &Signal {
+        &self.signal
+    }
+
+    pub fn new_block_notification(&self) -> Receiver<()> {
+        self.daemon.new_block_notification()
+    }
+
     pub fn sync(&mut self) -> Result<()> {
         self.tracker.sync(&self.daemon, self.signal.exit_flag())
     }
@@ -254,16 +262,16 @@ impl Rpc {
         (scripthash,): (ScriptHash,),
     ) -> Result<Value> {
         let history_entries = match client.scripthashes.get(&scripthash) {
-            Some(status) => self.tracker.get_history(status),
+            Some(status) => json!(status.get_history()),
             None => {
                 warn!(
                     "blockchain.scripthash.get_history called for unsubscribed scripthash: {}",
                     scripthash
                 );
-                self.tracker.get_history(&self.new_status(scripthash)?)
+                json!(self.new_status(scripthash)?.get_history())
             }
         };
-        Ok(json!(history_entries))
+        Ok(history_entries)
     }
 
     fn scripthash_list_unspent(
