@@ -1,4 +1,4 @@
-use anyhow::Result;
+use anyhow::{Context, Result};
 use bitcoin::consensus::{deserialize, serialize};
 use bitcoin::{Block, BlockHash, OutPoint, Txid};
 
@@ -6,7 +6,7 @@ use crate::{
     chain::Chain,
     daemon::Daemon,
     db::{DBStore, Row, WriteBatch},
-    metrics::{Gauge, Histogram, Metrics},
+    metrics::{self, Gauge, Histogram, Metrics},
     signals::ExitFlag,
     types::{HeaderRow, ScriptHash, ScriptHashRow, SpendingPrefixRow, TxidRow},
 };
@@ -16,6 +16,7 @@ struct Stats {
     update_duration: Histogram,
     update_size: Histogram,
     height: Gauge,
+    db_size: Gauge,
 }
 
 impl Stats {
@@ -25,13 +26,16 @@ impl Stats {
                 "index_update_duration",
                 "Index update duration (in seconds)",
                 "step",
+                metrics::default_duration_buckets(),
             ),
             update_size: metrics.histogram_vec(
                 "index_update_size",
                 "Index update size (in bytes)",
                 "step",
+                metrics::default_size_buckets(),
             ),
-            height: metrics.gauge("index_height", "Latest indexed block height"),
+            height: metrics.gauge("index_height", "Indexed block height", "type"),
+            db_size: metrics.gauge("index_db_size", "Index DB size (bytes)", "type"),
         }
     }
 }
@@ -75,6 +79,7 @@ impl Index {
         metrics: &Metrics,
         batch_size: usize,
         lookup_limit: Option<usize>,
+        reindex_last_blocks: usize,
     ) -> Result<Self> {
         if let Some(row) = store.get_tip() {
             let tip = deserialize(&row).expect("invalid tip");
@@ -84,14 +89,19 @@ impl Index {
                 .map(|row| HeaderRow::from_db_row(&row).header)
                 .collect();
             chain.load(headers, tip);
+            chain.drop_last_headers(reindex_last_blocks);
         };
 
+        let stats = Stats::new(metrics);
+        stats.height.set("tip", chain.height() as f64);
+        stats.db_size.set("total", store.get_size()? as f64);
+
         Ok(Index {
             store,
             batch_size,
             lookup_limit,
             chain,
-            stats: Stats::new(metrics),
+            stats,
         })
     }
 
@@ -161,7 +171,10 @@ impl Index {
     }
 
     pub(crate) fn sync(&mut self, daemon: &Daemon, exit_flag: &ExitFlag) -> Result<()> {
-        while !exit_flag.is_set() {
+        loop {
+            self.stats
+                .db_size
+                .set("total", self.store.get_size()? as f64);
             let new_headers =
                 self.observe_duration("headers", || daemon.get_new_headers(&self.chain))?;
             if new_headers.is_empty() {
@@ -174,12 +187,12 @@ impl Index {
                 new_headers.last().unwrap().height()
             );
             for chunk in new_headers.chunks(self.batch_size) {
-                if exit_flag.is_set() {
-                    bail!(
+                exit_flag.poll().with_context(|| {
+                    format!(
                         "indexing interrupted at height: {}",
                         chunk.first().unwrap().height()
                     )
-                }
+                })?;
                 let blockhashes: Vec<BlockHash> = chunk.iter().map(|h| h.hash()).collect();
                 let mut heights = chunk.iter().map(|h| h.height());
 
@@ -189,7 +202,7 @@ impl Index {
                     self.observe_duration("block", || {
                         index_single_block(block, height).extend(&mut batch)
                     });
-                    self.stats.height.set(height);
+                    self.stats.height.set("tip", height as f64);
                 })?;
                 let heights: Vec<_> = heights.collect();
                 assert!(
@@ -200,6 +213,9 @@ impl Index {
                 batch.sort();
                 self.report_stats(&batch);
                 self.observe_duration("write", || self.store.write(batch));
+                self.stats
+                    .db_size
+                    .set("total", self.store.get_size()? as f64);
             }
             self.chain.update(new_headers);
         }
