@@ -1,21 +1,17 @@
 use anyhow::{Context, Result};
-use bitcoin::BlockHash;
-use bitcoincore_rpc::RpcApi;
-use crossbeam_channel::{bounded, select, unbounded, Receiver, Sender};
+use crossbeam_channel::{select, unbounded, Sender};
 use rayon::prelude::*;
 
 use std::{
     collections::hash_map::HashMap,
-    convert::TryFrom,
     io::{BufRead, BufReader, Write},
     net::{Shutdown, TcpListener, TcpStream},
 };
 
 use crate::{
     config::Config,
-    daemon::rpc_connect,
     electrum::{Client, Rpc},
-    signals::Signal,
+    signals::ExitError,
     thread::spawn,
 };
 
@@ -43,89 +39,69 @@ impl Peer {
     }
 
     fn disconnect(self) {
-        let _ = self.stream.shutdown(Shutdown::Both);
-    }
-}
-
-fn handle_rpc_error(name: &str, err: bitcoincore_rpc::Error) -> Result<()> {
-    use bitcoincore_rpc::{
-        jsonrpc::{error::Error::Transport as TransportError, simple_http::Error as HttpError},
-        Error::JsonRpc as JsonRpcError,
-    };
-    if let JsonRpcError(TransportError(ref e)) = err {
-        if let Some(HttpError::Timeout) = e.downcast_ref::<HttpError>() {
-            // Following https://github.com/romanz/electrs/issues/495
-            warn!("ignoring HTTP timeout from RPC '{}'", name);
-            return Ok(());
+        if let Err(e) = self.stream.shutdown(Shutdown::Both) {
+            warn!("{}: failed to shutdown TCP connection {}", self.id, e)
         }
     }
-    bail!("RPC '{}' failed: {}", name, err); // fail on all other RPC errors
 }
 
-fn tip_receiver(config: &Config) -> Result<Receiver<BlockHash>> {
-    let duration = u64::try_from(config.wait_duration.as_millis()).unwrap();
-    let (tip_tx, tip_rx) = bounded(0);
-    let rpc = rpc_connect(config)?;
-
-    use crossbeam_channel::TrySendError;
-    spawn("tip_loop", move || loop {
-        let tip = match rpc.get_best_block_hash() {
-            Ok(tip) => tip,
-            Err(err) => {
-                handle_rpc_error("getbestblockhash", err)?;
-                continue;
+pub fn run() -> Result<()> {
+    let result = serve();
+    if let Err(e) = &result {
+        for cause in e.chain() {
+            if cause.downcast_ref::<ExitError>().is_some() {
+                info!("electrs stopped: {:?}", e);
+                return Ok(());
             }
-        };
-        match tip_tx.try_send(tip) {
-            Ok(_) | Err(TrySendError::Full(_)) => (),
-            Err(TrySendError::Disconnected(_)) => bail!("tip receiver disconnected"),
         }
-        if let Err(err) = rpc.wait_for_new_block(duration) {
-            warn!(
-                "waiting {:.1}s for new block failed: {}",
-                duration as f64 / 1e3,
-                err
-            );
-        }
-    });
-    Ok(tip_rx)
+    }
+    result.context("electrs failed")
 }
 
-pub fn run(config: &Config, mut rpc: Rpc) -> Result<()> {
-    let listener = TcpListener::bind(config.electrum_rpc_addr)?;
-    let tip_rx = tip_receiver(config)?;
-    info!("serving Electrum RPC on {}", listener.local_addr()?);
+fn serve() -> Result<()> {
+    let config = Config::from_args();
+    let mut rpc = Rpc::new(&config)?;
 
     let (server_tx, server_rx) = unbounded();
-    spawn("accept_loop", || accept_loop(listener, server_tx)); // detach accepting thread
-    let signal = Signal::new();
+    if !config.disable_electrum_rpc {
+        let listener = TcpListener::bind(config.electrum_rpc_addr)?;
+        info!("serving Electrum RPC on {}", listener.local_addr()?);
+        spawn("accept_loop", || accept_loop(listener, server_tx)); // detach accepting thread
+    };
 
+    let new_block_rx = rpc.new_block_notification();
     let mut peers = HashMap::<usize, Peer>::new();
     loop {
-        select! {
-            recv(signal.receiver()) -> result => {
-                result.context("signal channel disconnected")?;
-                if signal.exit_flag().is_set() {
-                    break;
-                }
-            },
-            recv(tip_rx) -> tip => match tip {
-                Ok(_) => (), // sync and update
-                Err(_) => break, // daemon is shutting down
-            },
-            recv(server_rx) -> event => {
-                let event = event.context("server disconnected")?;
-                let buffered_events = server_rx.iter().take(server_rx.len());
-                for event in std::iter::once(event).chain(buffered_events) {
-                    handle_event(&rpc, &mut peers, event);
-                }
-            },
-        };
-        rpc.sync().context("rpc sync failed")?;
+        rpc.sync().context("sync failed")?; // initial sync and compaction may take a few hours
+        if config.sync_once {
+            return Ok(());
+        }
         peers = notify_peers(&rpc, peers); // peers are disconnected on error.
+        loop {
+            select! {
+                // Handle signals for graceful shutdown
+                recv(rpc.signal().receiver()) -> result => {
+                    result.context("signal channel disconnected")?;
+                    rpc.signal().exit_flag().poll().context("RPC server interrupted")?;
+                },
+                // Handle new blocks' notifications
+                recv(new_block_rx) -> result => match result {
+                    Ok(_) => break, // sync and update
+                    Err(_) => return Ok(()), // daemon is shutting down
+                },
+                // Handle Electrum RPC requests
+                recv(server_rx) -> event => {
+                    let event = event.context("server disconnected")?;
+                    handle_event(&rpc, &mut peers, event);
+                },
+                default(config.wait_duration) => break, // sync and update
+            };
+            // continue RPC processing (if more requests are pending)
+            if server_rx.is_empty() {
+                break;
+            }
+        }
     }
-    info!("stopping Electrum RPC server");
-    Ok(())
 }
 
 fn notify_peers(rpc: &Rpc, peers: HashMap<usize, Peer>) -> HashMap<usize, Peer> {
@@ -196,7 +172,9 @@ fn accept_loop(listener: TcpListener, server_tx: Sender<Event>) -> Result<()> {
         let tx = server_tx.clone();
         spawn("recv_loop", move || {
             let result = recv_loop(peer_id, &stream, tx);
-            let _ = stream.shutdown(Shutdown::Both);
+            if let Err(e) = stream.shutdown(Shutdown::Read) {
+                warn!("{}: failed to shutdown TCP receiving {}", peer_id, e)
+            }
             result
         });
     }
